<?php
/**
 * Shopware 4
 * Copyright © shopware AG
 *
 * According to our dual licensing model, this program can be used either
 * under the terms of the GNU Affero General Public License, version 3,
 * or under a proprietary license.
 *
 * The texts of the GNU Affero General Public License with an additional
 * permission and of our proprietary license can be found at and
 * in the LICENSE file you have received along with this program.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * "Shopware" is a registered trademark of shopware AG.
 * The licensing of the program under the AGPLv3 does not imply a
 * trademark license. Therefore any rights, title and interest in
 * our trademarks remain entirely with us.
 */
namespace Shopware\CustomModels\eTracker;

use Shopware\Components\Model\ModelEntity;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * @ORM\Entity
 * @ORM\Table(name="s_etracker_configs")
 */
class Config extends ModelEntity
{
	/**
	 * @var integer $id
	 *
	 * @ORM\Id
	 * @ORM\Column(type="integer")
	 * @ORM\GeneratedValue(strategy="IDENTITY")
	 */
	private $id;

	/**
	 * @var integer $storeId
	 *
	 * @ORM\Column(name="store_id", type="integer", nullable=false)
	 */
	private $storeId;

	/**
	 * @var string $accountKey
	 *
	 * @ORM\Column(name="account_key", type="string", nullable=false)
	 */
	private $accountKey;

	/**
	 * @var integer $useNetPrice
	 *
	 * @ORM\Column(name="net_price", type="integer", nullable=false)
	 */
	private $useNetPrice;

	/**
	 * @var boolean $ownPageNames
	 *
	 * @ORM\Column(name="own_page_names", type="boolean", nullable=false)
	 */
	private $ownPageNames = true;

	/**
	 * @var integer $shopwareDefault
	 *
	 * @ORM\Column(name="shopware_default", type="integer", nullable=false)
	 */
	private $shopwareDefault;

	/**
	 * @var string $ownSchema
	 *
	 * @ORM\Column(name="own_schema", type="string", nullable=false)
	 */
	private $ownSchema = false;

	/**
	 * @var integer $divideAreas
	 *
	 * @ORM\Column(name="divide_areas", type="integer", nullable=false)
	 */
	private $divideAreas;

	/**
	 * INVERSE SIDE
	 *
	 * @var ArrayCollection $interests
	 *
	 * @ORM\OneToMany(targetEntity="Shopware\CustomModels\eTracker\Interest", mappedBy="config", cascade={"persist"})
	 * @ORM\OrderBy({"level" = "ASC"})
	 */
	private $interests;

	/**
	 * @return int
	 */
	public function getId()
	{
		return $this->id;
	}

	/**
	 * @param int $storeId
	 * @return $this
	 */
	public function setStoreId($storeId)
	{
		$this->storeId = $storeId;
		return $this;
	}

	/**
	 * @return int
	 */
	public function getStoreId()
	{
		return $this->storeId;
	}


	/**
	 * @param string $accountKey
	 * @return $this
	 */
	public function setAccountKey($accountKey)
	{
		$this->accountKey = $accountKey;
		return $this;
	}

	/**
	 * @return string
	 */
	public function getAccountKey()
	{
		return $this->accountKey;
	}

	/**
	 * @param int $useNetPrice
	 * @return $this
	 */
	public function setUseNetPrice($useNetPrice)
	{
		$this->useNetPrice = $useNetPrice;
		return $this;
	}

	/**
	 * @return int
	 */
	public function getUseNetPrice()
	{
		return $this->useNetPrice;
	}

	/**
	 * @param boolean $ownPageNames
	 * @return $this
	 */
	public function setOwnPageNames($ownPageNames)
	{
		$this->ownPageNames = $ownPageNames;
		return $this;
	}

	/**
	 * @return boolean
	 */
	public function getOwnPageNames()
	{
		return $this->ownPageNames;
	}

	/**
	 * @param string $ownSchema
	 * @return $this
	 */
	public function setOwnSchema($ownSchema)
	{
		$this->ownSchema = $ownSchema;
		return $this;
	}

	/**
	 * @return string
	 */
	public function getOwnSchema()
	{
		return $this->ownSchema;
	}

	/**
	 * @param int $divideAreas
	 * @return $this
	 */
	public function setDivideAreas($divideAreas)
	{
		$this->divideAreas = $divideAreas;
		return $this;
	}

	/**
	 * @return int
	 */
	public function getDivideAreas()
	{
		return $this->divideAreas;
	}

	/**
	 * @param integer $shopwareDefault
	 * @return $this
	 */
	public function setShopwareDefault($shopwareDefault)
	{
		$this->shopwareDefault = $shopwareDefault;
		return $this;
	}

	/**
	 * @return integer
	 */
	public function getShopwareDefault()
	{
		return $this->shopwareDefault;
	}

	/**
	 * @param \Doctrine\Common\Collections\ArrayCollection $interests
	 * @return $this
	 */
	public function setInterests($interests)
	{
		$this->interests = $interests;
		return $this;
	}

	/**
	 * @return \Doctrine\Common\Collections\ArrayCollection
	 */
	public function getInterests()
	{
		return $this->interests;
	}

	public function addInterest($interest)
	{
		$this->interests->add($interest);
	}
}