<?php
/**
 * Shopware 4
 * Copyright © shopware AG
 *
 * According to our dual licensing model, this program can be used either
 * under the terms of the GNU Affero General Public License, version 3,
 * or under a proprietary license.
 *
 * The texts of the GNU Affero General Public License with an additional
 * permission and of our proprietary license can be found at and
 * in the LICENSE file you have received along with this program.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * "Shopware" is a registered trademark of shopware AG.
 * The licensing of the program under the AGPLv3 does not imply a
 * trademark license. Therefore any rights, title and interest in
 * our trademarks remain entirely with us.
 */
namespace Shopware\CustomModels\eTracker;

use Shopware\Components\Model\ModelEntity;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * @ORM\Entity
 * @ORM\Table(name="s_etracker_interest")
 */
class Interest extends ModelEntity
{
	/**
	 * @var integer $id
	 *
	 * @ORM\Id
	 * @ORM\Column(type="integer")
	 * @ORM\GeneratedValue(strategy="IDENTITY")
	 */
	private $id;

	/**
	 * @var string $name
	 *
	 * @ORM\Column(name="name", type="string", nullable=false)
	 */
	private $name;

	/**
	 * @var integer $level
	 *
	 * @ORM\Column(name="level", type="integer", nullable=false)
	 */
	private $level;

	/**
	 * @var integer $controllerId
	 *
	 * @ORM\Column(name="controller_id", type="integer", nullable=false)
	 */
	private $controllerId;

	/**
	 * @var \Shopware\CustomModels\eTracker\Controller $controller
	 * @ORM\OneToOne(targetEntity="Shopware\CustomModels\eTracker\Controller")
	 * @ORM\JoinColumn(name="controller_id", referencedColumnName="id")
	 */
	private $controller;

	/**
	 * @var integer $configId
	 *
	 * @ORM\Column(name="config_id", type="integer", nullable=false)
	 */
	private $configId;

	/**
	 * OWNING SIDE
	 * @ORM\ManyToOne(targetEntity="Shopware\CustomModels\eTracker\Config", inversedBy="interests")
	 * @ORM\JoinColumn(name="config_id", referencedColumnName="id")
	 * @ORM\OrderBy({"level" = "ASC"})
	 */
	private $config;

	/**
	 * @return integer
	 */
	public function getId()
	{
		return $this->id;
	}

	/**
	 * @param string $name
	 * @return $this
	 */
	public function setName($name)
	{
		$this->name = $name;
		return $this;
	}

	/**
	 * @return string
	 */
	public function getName()
	{
		return $this->name;
	}

	/**
	 * @param string $level
	 * @return $this
	 */
	public function setLevel($level)
	{
		$this->level = $level;
		return $this;
	}

	/**
	 * @return string
	 */
	public function getLevel()
	{
		return $this->level;
	}

	/**
	 * @param \Shopware\CustomModels\eTracker\Controller $controller
	 * @return $this
	 */
	public function setController($controller)
	{
		$this->controller = $controller;
		return $this;
	}

	/**
	 * @return \Shopware\CustomModels\eTracker\Controller
	 */
	public function getController()
	{
		return $this->controller;
	}

	/**
	 * @param mixed $controllerId
	 * @return $this
	 */
	public function setControllerId($controllerId)
	{
		$this->controllerId = $controllerId;
		return $this;
	}

	/**
	 * @return mixed
	 */
	public function getControllerId()
	{
		return $this->controllerId;
	}

	/**
	 * @param mixed $config
	 * @return $this
	 */
	public function setConfig($config)
	{
		$this->config = $config;
		return $this;
	}

	/**
	 * @return mixed
	 */
	public function getConfig()
	{
		return $this->config;
	}

	/**
	 * @param integer $configId
	 * @return $this
	 */
	public function setConfigId($configId)
	{
		$this->configId = $configId;
		return $this;
	}

	/**
	 * @return integer
	 */
	public function getConfigId()
	{
		return $this->configId;
	}
}