/**
 * Shopware 4
 * Copyright © shopware AG
 *
 * According to our dual licensing model, this program can be used either
 * under the terms of the GNU Affero General Public License, version 3,
 * or under a proprietary license.
 *
 * The texts of the GNU Affero General Public License with an additional
 * permission and of our proprietary license can be found at and
 * in the LICENSE file you have received along with this program.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * "Shopware" is a registered trademark of shopware AG.
 * The licensing of the program under the AGPLv3 does not imply a
 * trademark license. Therefore any rights, title and interest in
 * our trademarks remain entirely with us.
 *
 * @category   Shopware
 * @package    SwagETracker
 * @subpackage View
 * @copyright  Copyright © shopware AG (http://www.shopware.de)
 * @version    $Id$
 * @author shopware AG
 */

//{namespace name=backend/etracker/view/main}
//{block name="backend/swag_e_tracker/view/tracker/interested_detail"}
Ext.define('Shopware.apps.SwagETracker.view.tracker.InterestDetail', {
    extend: 'Enlight.app.Window',
    alias: 'widget.swagetracker-tracker-interest_detail',
    cls: 'createWindow',
    modal: true,
    layout: {
        type: 'vbox',
        align: 'stretch',
        pack: 'start'
    },
    defaults: {
        flex: 1
    },
    autoShow: true,
    title: '{s name="interestDetail/windowTitle"}Interest detail{/s}',
    border: 0,
    width: 600,
    height: 200,
    stateful: true,
    stateId: 'shopware-tracker-interest_detail',
    footerButton: false,

    snippets: {
        buttons: {
            cancel: '{s name="view/buttons/cancel"}Cancel{/s}',
            save: '{s name="view/buttons/save"}Save{/s}'
        },
        form: {
            name: {
                fieldLabel: '{s name="view/interestDetail/name/fieldLabel"}Name{/s}'
            },
            level: {
                fieldLabel: '{s name="view/interestDetail/level/fieldLabel"}Level{/s}'
            },
            controller: {
                fieldLabel: '{s name="view/interestDetail/controller/fieldLabel"}Controller{/s}',
                emptyText: '{s name="view/interestDetail/controller/emptyText"}Search{/s}'
            }
        }
    },

    /**
     * This method is called when this view is initiated.
     * It handles the creation of all the important components.
     */
    initComponent:function () {
        var me = this;

        me.interestDetailForm = me.createFormPanel();
        me.dockedItems = [{
            xtype: 'toolbar',
            ui: 'shopware-ui',
            dock: 'bottom',
            cls: 'shopware-toolbar',
            items: me.createButtons()
        }];

        me.items = [ me.interestDetailForm ];
        me.callParent(arguments);
    },

    /**
     * This method creates the form-panel with three simple fields.
     *
     * @returns Ext.form.Panel
     */
    createFormPanel: function(){
        var me = this,
            searchField = Ext.create('Shopware.form.field.Search', {
                action : 'searchDivisions',
                width : 170,
                enableKeyEvents : true,
                emptyText : me.snippets.form.controller.emptyText,
                store: Ext.create('Shopware.apps.SwagETracker.store.Controllers'),
                displayField: "name",
                valueField: 'id',
                fieldLabel: me.snippets.form.controller.fieldLabel,
                labelWidth:200,
                anchor: '100%',
                name: 'controllerId'
            }),
            hiddenField = Ext.create('Ext.form.field.Hidden', {
                name: 'configId'
            }),
            formPanel = Ext.create('Ext.form.Panel', {
                layout: 'anchor',
                defaults:{
                    labelWidth:200,
                    anchor: '100%'
                },
                bodyPadding: 18,
                items:[
                    {
                        xtype: 'hidden',
                        name: 'id'
                    },
                    hiddenField,
                    {
                        xtype: 'textfield',
                        fieldLabel: me.snippets.form.name.fieldLabel,
                        name: 'name'
                    },
                    {
                        xtype: 'numberfield',
                        fieldLabel: me.snippets.form.level.fieldLabel,
                        name: 'level',
                        minValue: 1
                    },
                    searchField
                ]
            });

        if(me.interestRecord){
            formPanel.loadRecord(me.interestRecord);
            searchField.setValue(me.interestRecord.getController().getAt(0).get('name'));
        }

        if(me.configId)
        {
            hiddenField.setValue(me.configId);
        }

        return formPanel;
    },

    /**
     * This method creates the save-/cancel-button
     * @returns Array
     */
    createButtons: function(){
        var me = this;
        var buttons = ['->',
            {
                text: me.snippets.buttons.cancel,
                cls: 'secondary',
                scope:me,
                handler:me.destroy
            },
            {
                text: me.snippets.buttons.save,
                action:'saveInterest',
                cls:'primary'
            }
        ];

        return buttons;
    }
});
//{/block}