/**
 * Shopware 4
 * Copyright © shopware AG
 *
 * According to our dual licensing model, this program can be used either
 * under the terms of the GNU Affero General Public License, version 3,
 * or under a proprietary license.
 *
 * The texts of the GNU Affero General Public License with an additional
 * permission and of our proprietary license can be found at and
 * in the LICENSE file you have received along with this program.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * "Shopware" is a registered trademark of shopware AG.
 * The licensing of the program under the AGPLv3 does not imply a
 * trademark license. Therefore any rights, title and interest in
 * our trademarks remain entirely with us.
 *
 * @category   Shopware
 * @package    SwagETracker
 * @subpackage View
 * @copyright  Copyright © shopware AG (http://www.shopware.de)
 * @version    $Id$
 * @author shopware AG
 */

//{namespace name=backend/etracker/view/main}
//{block name="backend/swag_e_tracker/view/tracker/settings"}
Ext.define('Shopware.apps.SwagETracker.view.tracker.Settings', {

    /**
     * Extend from the standard ExtJS 4
     * @string
     */
    extend: 'Ext.form.Panel',
    
    cls: 'shopware-form',
    autoScroll: true,

    bodyPadding: 10,

    defaults:{
        labelWidth: 200
    },
    border: 0,

    snippets: {
        toolTips: {
            edit: '{s name="view/toolTips/edit"}Edit entry{/s}',
            delete: '{s name="view/toolTips/delete"}Delete entry{/s}'
        },
        buttons: {
            add: '{s name="view/settings/addButton"}Add interest{/s}',
            remove:'{s name="view/settings/removeButton"}Remove interests{/s}',
            cancel: '{s name="view/buttons/cancel"}Cancel{/s}',
            save: '{s name="view/buttons/save"}Save{/s}'
        },
        form: {
            accountKey: {
                fieldLabel: '{s name="view/settings/accountKey/fieldLabel"}Account-Key{/s}'
            },
            testAccount: {
                html: '{s name="view/settings/testAccount/html"}<div style="text-align: right;">30 Tage <span style="font-weight: bold;">kostenloser</span> Test: <a href="https://www.connectiv.de/etracker" target="_blank">Account-Schlüssel anfordern</a></div>{/s}'
            },
            useNetPrice: {
                fieldLabel: '{s name="view/settings/useNetPrice/fieldLabel"}Use gross-prices{/s}'
            },
            ownPageNames: {
                title: '{s name="view/settings/ownPageNames/title"}Use own pagenames{/s}'
            },
            shopwareDefault: {
                fieldLabel: '{s name="view/settings/shopwareDefault/fieldLabel"}Choose{/s}',
                emptyText: '{s name="view/settings/shopwareDefault/emptyText"}Please choose{/s}',
                options: {
                    shopwareDefault: '{s name="view/settings/shopwareDefault/options/shopwareDefault"}Use shopware default{/s}',
                    ownSchema: '{s name="view/settings/shopwareDefault/options/ownSchema"}Use own schema{/s}'
                }
            },
            ownSchema: {
                fieldLabel: '{s name="view/settings/ownSchema/fieldLabel"}Schema{/s}',
                emptyText: '{s name="view/settings/ownSchema/emptyText"}%controller%_%action%{/s}',
                helpText: '{s name="view/settings/ownSchema/helpText"}Possible values: %controller%, %action%, %id%, %custom%{/s}'
            },
            divideAreas: {
                fieldLabel: '{s name="view/settings/divideAreas/fieldLabel"}Divide areas into{/s}',
                emptyText: '{s name="view/settings/divideAreas/fieldLabel"}Please choose{/s}',
                options: {
                    default: '{s name="view/settings/divideAreas/options/default"}Controller{/s}',
                    controller: '{s name="view/settings/divideAreas/options/controller"}Controller{/s}',
                    categories: '{s name="view/settings/divideAreas/options/categories"}Categories{/s}',
                    shops: '{s name="view/settings/divideAreas/options/shops"}Shops{/s}',
                    ownArea: '{s name="view/settings/divideAreas/options/ownArea"}Own areas{/s}'
                }
            }
        },
        interestGrid: {
            title: '{s name="view/settings/interestGrid/title"}Interest-levels{/s}',
            columns: {
                name: '{s name="view/settings/interestGrid/columns/name"}Name{/s}',
                controller: '{s name="view/settings/interestGrid/columns/controller"}Controller{/s}',
                level: '{s name="view/settings/interestGrid/columns/level"}Level{/s}'
            }
        }
    },

    /**
     * Alias name for the view. Could be used to get an instance
     * of the view through Ext.widget('swagetracker-tracker-settings')
     * @string
     */
    alias: 'widget.swagetracker-tracker-settings',

    /**
     * This method is called when this view is initiated.
     * It handles the creation of all the important components.
     */
    initComponent: function(){
        var me = this;

        me.registerEvents();

        me.dockedItems = [{
            xtype: 'toolbar',
            ui: 'shopware-ui',
            dock: 'bottom',
            cls: 'shopware-toolbar',
            items: me.createButtons()
        }];

        me.items = me.getItems();
        
        this.callParent(arguments);
    },

    /**
     * Defines additional events which will be
     * fired from the component
     *
     * @return void
     */
    registerEvents: function(){
        var me = this;

        me.addEvents(

                /**
                 * Event will be fired when the user changes the value of the area-dropdown
                 *
                 * @event changeAreaCombo
                 * @param [object] comp - The combobox, which triggered the event
                 */
                'changeAreaCombo',

                /**
                 * Event will be fired when the user changes the value of the schema-dropdown
                 *
                 * @event changeSchemaCombo
                 * @param [object] comp - The combobox, which triggered the event
                 */
                'changeSchemaCombo'
        );

        return true;
    },

    /**
     * This method creates the form-panel with every needed field(set).
     * It also creates the interest-grid and the buttons to save or cancel the form.
     *
     * @returns Array
     */
    getItems: function(){
        var me = this,
            configForm = me.getConfigForm(),
            interestGrid = me.getInterestGrid();

        if (me.configRecord) {
            configForm.loadRecord(me.configRecord);
            var interestStore = Ext.create('Shopware.apps.SwagETracker.store.Interests');
            interestStore.loadRecords(me.configRecord.getInterests().data.items);
            interestGrid.reconfigure(interestStore);
        }

        return [configForm, interestGrid];
    },

    /**
     * Helper method to render the controller-column in the interest-grid.
     * @param value
     * @param metaData
     * @param record
     * @returns string
     */
    renderControllerColumn: function(value, metaData, record){
        return record.getController().getAt(0).get('name');
    },

    /**
     * Helper method to create the save-/cancel-button
     * @returns Array
     */
    createButtons: function(){
        var me = this;
        var buttons = ['->',
            {
                text: me.snippets.buttons.cancel,
                cls: 'secondary',
                scope: me,
                handler: me.destroy
            },
            {
                text: me.snippets.buttons.save,
                action:'saveConfig',
                cls:'primary'
            }
        ];

        return buttons;
    },

    /**
     * Helper method to get the config form
     * @returns Ext.form.Panel
     */
    getConfigForm: function() {
        var me = this;
        return Ext.create('Ext.form.Panel', {
            border: 0,
            items: [
                {
                    xtype: 'hidden',
                    name: 'id'
                },
                {
                    xtype: 'textfield',
                    name: 'accountKey',
                    fieldLabel: me.snippets.form.accountKey.fieldLabel,
                    labelWidth: 180,
                    anchor:'100%',
                },{
                    xtype: 'panel',
                    html: me.snippets.form.testAccount.html,
                    fieldLabel: '',
                    labelWidth: 180,
                    anchor:'100%',
                    border: 0
                }, {
                    xtype: 'checkbox',
                    name: 'useNetPrice',
                    inputValue: 1,
                    uncheckedValue: 0,
                    fieldLabel: me.snippets.form.useNetPrice.fieldLabel,
                    labelWidth: 180,
                    anchor:'100%'
                }, {
                    xtype: 'fieldset',
                    name: 'pagesFieldSet',
                    checkboxToggle: true,
                    checkboxName: 'ownPageNames',
                    title: me.snippets.form.ownPageNames.title,
                    items: me.getFieldSetItems()
                }, {
                    xtype: 'combo',
                    fieldLabel: me.snippets.form.divideAreas.fieldLabel,
                    emptyText: me.snippets.form.divideAreas.emptyText,
                    checkChangeBuffer: 1500,
                    displayField: 'name',
                    valueField: 'areaId',
                    name: 'divideAreas',
                    labelWidth: 180,
                    anchor:'100%',
                    store: Ext.create('Ext.data.Store', {
                        fields: ['areaId', 'name', 'eventField'],
                        data : [
                            { areaId: 4, name: me.snippets.form.divideAreas.options.default, eventField: 0 },
                            { areaId: 0, name: me.snippets.form.divideAreas.options.controller, eventField: 0 },
                            { areaId: 1, name: me.snippets.form.divideAreas.options.categories, eventField: 0 },
                            { areaId: 2, name: me.snippets.form.divideAreas.options.shops, eventField: 0 },
                            { areaId: 3, name: me.snippets.form.divideAreas.options.ownArea, eventField: 1 }
                        ]
                    }),
                    listeners: {
                        change: {
                            fn: function(comp){
                                me.fireEvent('changeAreaCombo', comp);
                            }
                        }
                    }
                }
            ]
        });
    },

    /**
     * Helper method to get all the buttons for the action-column
     * @returns Array
     */
    getActionButtons: function(){
        var me = this,
            buttons = [];
        buttons.push(Ext.create('Ext.button.Button', {
            iconCls: 'sprite-minus-circle',
            action: 'delete',
            cls: 'delete',
            tooltip: me.snippets.toolTips.delete,
            handler:function (view, rowIndex, colIndex, item) {
                me.fireEvent('deleteInterestColumn', view, rowIndex,  item, colIndex);
            }
        }));

        buttons.push(Ext.create('Ext.button.Button', {
            iconCls: 'sprite-pencil',
            cls: 'editBtn',
            tooltip: me.snippets.toolTips.edit,
            handler:function (view, rowIndex, colIndex, item) {
                me.fireEvent('editInterestColumn', view, item, rowIndex, colIndex);
            }
        }));

        return buttons;
    },

    /**
     * Helper method to get all the buttons for the toolbar
     * @returns Array
     */
    getToolBarButtons: function(){
        var me = this,
            items = [];
        items.push(Ext.create('Ext.button.Button',{
            iconCls: 'sprite-plus-circle',
            text: me.snippets.buttons.add,
            action: 'add'
        }));

        items.push(Ext.create('Ext.button.Button',{
            iconCls: 'sprite-minus-circle',
            text: me.snippets.buttons.remove,
            disabled: true,
            action: 'deleteMultipleInterests'
        }));

        return items;
    },

    /**
     * Helper method to get the toolbar itself
     * @returns Ext.toolbar.Toolbar
     */
    getToolBar: function(){
        var me = this;
        return Ext.create('Ext.toolbar.Toolbar', {
            dock: 'top',
            ui: 'shopware-ui',
            items: me.getToolBarButtons()
        });
    },

    /**
     * Helper method to create the interestGrid
     * @returns Ext.grid.Panel
     */
    getInterestGrid: function(){
        var me = this;
        return Ext.create('Ext.grid.Panel', {
            title: me.snippets.interestGrid.title,
            store: Ext.create('Shopware.apps.SwagETracker.store.Interests'),
            dockedItems: [me.getToolBar()],
            selModel: Ext.create('Ext.selection.CheckboxModel',{
                listeners: {
                    selectionchange: function(sm, selections){
                        var owner = this.view.ownerCt,
                                btn = owner.down('button[action=deleteMultipleInterests]');

                        //If no article is marked
                        if(btn){
                            btn.setDisabled(selections.length == 0);
                        }
                    }
                }
            }),
            columns: [
                { text: 'id',  dataIndex: 'id', hidden: true },
                { text: me.snippets.interestGrid.columns.name, dataIndex: 'name', flex: 2 },
                { text: me.snippets.interestGrid.columns.controller, dataIndex: 'controllerId', renderer: me.renderControllerColumn },
                { text: me.snippets.interestGrid.columns.level, dataIndex: 'level' },
                {
                    xtype: 'actioncolumn',
                    width: 60,
                    items: me.getActionButtons()
                }
            ]
        });
    },

    /**
     * Helper method to get all the items needed in the fieldset
     * @returns Array
     */
    getFieldSetItems: function(){
        var me = this;
        return [
            {
                xtype: 'combo',
                align: 'right',
                fieldLabel: me.snippets.form.shopwareDefault.fieldLabel,
                displayField: 'display',
                valueField: 'value',
                emptyText: me.snippets.form.shopwareDefault.emptyText,
                labelWidth: 180,
                inputWidth: 200,
                value: 0,
                name: 'shopwareDefault',
                store: Ext.create('Ext.data.Store', {
                    fields: ['value', 'display', 'eventField'],
                    data : [
                        { value: 0, display: me.snippets.form.shopwareDefault.options.shopwareDefault, eventField: 0 },
                        { value: 1, display: me.snippets.form.shopwareDefault.options.ownSchema, eventField: 1 }
                    ]
                }),
                listeners: {
                    change: {
                        fn: function(comp){
                            me.fireEvent('changeSchemaCombo', comp);
                        }
                    }
                }
            },{
            xtype: 'textfield',
            fieldLabel: me.snippets.form.ownSchema.fieldLabel,
            labelWidth: 180,
            emptyText: me.snippets.form.ownSchema.emptyText,
            name: 'ownSchema',
            inputWidth: 200,
            disabled: true,
            helpText: me.snippets.form.ownSchema.helpText
        }
        ]
    }
});
//{/block}